from typing import Optional

from pydantic import field_validator

from galtea.utils.from_camel_case_base_model import FromCamelCaseBaseModel


class GenerateNextTurnRequest(FromCamelCaseBaseModel):
    session_id: str
    max_turns: Optional[int]


class GenerateNextTurnResponse(FromCamelCaseBaseModel):
    next_message: str
    finished: bool
    stopping_reason: Optional[str] = None
    simulator_version: Optional[str] = None
    inference_result_id: str

    @field_validator("simulator_version", mode="before")
    @classmethod
    def convert_simulator_version_to_string(cls, v):
        """Convert simulator_version to string if it's an integer."""
        if v is not None and not isinstance(v, str):
            return str(v)
        return v



from typing import Optional
from warnings import warn

from pydantic import Field

from galtea.utils.from_camel_case_base_model import FromCamelCaseBaseModel


class EvaluationTask(FromCamelCaseBaseModel):
    """
    **DEPRECATED**
    Use `Evaluation` class instead.
    """

    id: str
    metric_type_id: str = Field(alias="metricId")
    score: Optional[float] = None
    status: str
    conversation_simulator_version: Optional[str] = None
    created_at: str
    deleted_at: Optional[str] = None
    evaluated_at: Optional[str] = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        warn("EvaluationTask is deprecated and will be removed in future versions.", DeprecationWarning, stacklevel=2)



from typing import List, Optional

from galtea.domain.exceptions.entity_not_found_exception import EntityNotFoundException
from galtea.domain.models.product import Product
from galtea.infrastructure.clients.http_client import Client
from galtea.utils.string import build_query_params, is_valid_id


class ProductService:
    def __init__(self, client: Client):
        self.__client = client

    def get(self, product_id: str):
        """
        Retrieve a product by its ID.

        Args:
            product_id (str): ID of the product to retrieve.

        Returns:
            Product: The retrieved product object.
        """
        if not is_valid_id(product_id):
            raise ValueError("Product ID provided is not valid.")

        response = self.__client.get(f"products/{product_id}")
        return Product(**response.json())

    def get_by_name(self, name: str):
        """
        Retrieve a product by its name.

        Args:
            name (str): Name of the product to retrieve.

        Returns:
            Product: The retrieved product object.
        """
        query_params = build_query_params(names=[name])
        response = self.__client.get(f"products?{query_params}")
        products = [Product(**product) for product in response.json()]

        if not products:
            raise EntityNotFoundException(f"Product with name {name} does not exist.")

        return products[0]

    def delete(self, product_id: str):
        """
        Delete a product by its ID.

        Args:
            product_id (str): ID of the product to delete.

        Returns:
            None: None.
        """
        if not is_valid_id(product_id):
            raise ValueError("Product ID provided is not valid.")

        self.__client.delete(f"products/{product_id}")

    def list(
        self, offset: Optional[int] = None, limit: Optional[int] = None, sort_by_created_at: Optional[str] = None
    ) -> List[Product]:
        """
        Get a list of products.

        Args:
            offset (int, optional): Offset for pagination.
            limit (int, optional): Limit for pagination.
            sort_by_created_at (str, optional): Sort by created at. Valid values are 'asc' and 'desc'.

        Returns:
            list[Product]: List of products.
        """
        if sort_by_created_at is not None and sort_by_created_at not in ["asc", "desc"]:
            raise ValueError("Sort by created at must be 'asc' or 'desc'.")

        query_params = build_query_params(
            offset=offset,
            limit=limit,
            sort=["createdAt", sort_by_created_at] if sort_by_created_at else None,
        )
        response = self.__client.get(f"products?{query_params}")
        products = [Product(**product) for product in response.json()]
        return products



from typing import Dict, List, Optional, Union

from galtea.application.services.evaluation_service import EvaluationService
from galtea.application.services.metric_type_service import MetricTypeService
from galtea.application.services.session_service import SessionService
from galtea.application.services.test_case_service import TestCaseService
from galtea.domain.models.evaluation_task import EvaluationTask
from galtea.infrastructure.clients.http_client import Client
from galtea.utils.custom_score_metric import CustomScoreEvaluationMetric


class EvaluationTaskService:
    """
    Service for managing Evaluation Tasks.
    An Evaluation Task is the result of an evaluation against a specific metric and its criteria.
    Evaluations are created implicitly when evaluation tasks are created.
    """

    def __init__(
        self,
        client: Client,
        evaluation_service: EvaluationService,
        metric_type_service: MetricTypeService,
        session_service: SessionService,
        test_case_service: TestCaseService,
    ):
        self.__client = client
        self.__evaluation_service = evaluation_service
        self.__metric_type_service = metric_type_service
        self.__session_service = session_service
        self.__test_case_service = test_case_service

    def create_single_turn(
        self,
        version_id: str,
        actual_output: str,
        metrics: Optional[List[Union[str, CustomScoreEvaluationMetric]]] = None,
        metric_ids: Optional[List[str]] = None,
        test_case_id: Optional[str] = None,
        input: Optional[str] = None,
        is_production: Optional[bool] = None,
        retrieval_context: Optional[str] = None,
        latency: Optional[float] = None,
        usage_info: Optional[Dict[str, float]] = None,
        cost_info: Optional[Dict[str, float]] = None,
        conversation_simulator_version: Optional[str] = None,
    ) -> Optional[List[EvaluationTask]]:
        """
        Deprecated: The concept of "EvaluationTask" has been renamed to "Evaluation". Use "galtea.evaluations.create_single_turn" instead.
        """
        raise NotImplementedError(
            "The concept of 'EvaluationTask' has been renamed to 'Evaluation'. "
            "Use 'galtea.evaluations.create_single_turn' instead."
        )

    def create(
        self,
        metrics: List[str],
        session_id: str,
    ) -> List[EvaluationTask]:
        """
        Deprecated: The concept of "EvaluationTask" has been renamed to "Evaluation". Use "galtea.evaluations.create" instead.
        """
        raise NotImplementedError(
            "The concept of 'EvaluationTask' has been renamed to 'Evaluation'. Use 'galtea.evaluations.create' instead."
        )

    def get(self, evaluation_task_id: str):
        """
        Deprecated: The concept of "EvaluationTask" has been renamed to "Evaluation".
        Use "galtea.evaluations.get" instead.
        """
        raise NotImplementedError(
            "The concept of 'EvaluationTask' has been renamed to 'Evaluation'. Use 'galtea.evaluations.get' instead."
        )

    def list(
        self,
        session_id: Optional[str] = None,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
    ):
        """
        Deprecated: The concept of "EvaluationTask" has been renamed to "Evaluation".
        Use "galtea.evaluations.list" instead.
        """
        raise NotImplementedError(
            "The concept of 'EvaluationTask' has been renamed to 'Evaluation'. Use 'galtea.evaluations.list' instead."
        )

    def delete(self, evaluation_task_id: str):
        """
        Deprecated: The concept of "EvaluationTask" has been renamed to "Evaluation".
        Use "galtea.evaluations.delete" instead.
        """
        raise NotImplementedError(
            "The concept of 'EvaluationTask' has been renamed to 'Evaluation'. Use 'galtea.evaluations.delete' instead."
        )



from typing import Optional

from galtea.utils.from_camel_case_base_model import FromCamelCaseBaseModel


class TestCaseBase(FromCamelCaseBaseModel):
    """
    Base model for a test case.

    Attributes:
        test_id (str): ID of the test.
        input (str): Input for the test case.
        expected_output (Optional[str]): Expected output for the test case.
        context (Optional[str]): Context for the test case.
        source (Optional[str]): Source of the test case.
        strategy (Optional[str]): Strategy for the test case.
        variant (Optional[str]): Variant for the test case.
        reviewed_by_id (Optional[str]): ID of the user who reviewed the test case.
        language_code (Optional[str]): Language code for the test case.
        user_score (Optional[int]): User score for the test case.
        user_score_reason (Optional[str]): User score reason for the test case.
    """

    test_id: str

    # Quality / Red Teaming Test Case
    input: Optional[str] = None
    expected_output: Optional[str] = None
    context: Optional[str] = None
    variant: Optional[str] = None
    strategy: Optional[str] = None

    # SCENARIO Test Case
    user_persona: Optional[str] = None
    scenario: Optional[str] = None
    goal: Optional[str] = None
    max_iterations: Optional[int] = None
    initial_prompt: Optional[str] = None
    stopping_criterias: Optional[list[str]] = None

    # COMMON FIELDS
    source: Optional[str] = None
    reviewed_by_id: Optional[str] = None
    language_code: Optional[str] = None
    user_score: Optional[int] = None
    user_score_reason: Optional[str] = None
    confidence: Optional[float] = None
    confidence_reason: Optional[str] = None


class TestCase(TestCaseBase):
    """
    Model for a test case, including database identifiers and timestamps.

    Attributes:
        id (str): Unique identifier for the test case.
        created_at (str): Creation timestamp.
        deleted_at (Optional[str]): Deletion timestamp, if deleted.
    """

    id: str
    created_at: str
    deleted_at: Optional[str] = None
